# Copyright (C) 2008-2009 Adam Olsen 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#
# The developers of the Exaile media player hereby grant permission 
# for non-GPL compatible GStreamer and Exaile plugins to be used and 
# distributed together with GStreamer and Exaile. This permission is 
# above and beyond the permissions granted by the GPL license by which 
# Exaile is covered. If you modify this code, you may extend this 
# exception to your version of the code, but you are not obligated to 
# do so. If you do not wish to do so, delete this exception statement 
# from your version.

from xl.metadata import BaseFormat
from mutagen import FileType

import os

try:
    import ctypes
    modplug = ctypes.cdll.LoadLibrary("libmodplug.so.0")
    modplug.ModPlug_GetName.restype = ctypes.c_char_p
except (ImportError, OSError):
    modplug = None

class ModFormat(BaseFormat):
    MutagenType = FileType #not actually used

    def load(self):
        if modplug:
            data = open(self.loc, "rb").read()
            f = modplug.ModPlug_Load(data, len(data))
            if f:
                name = modplug.ModPlug_GetName(f) or os.path.split(self.loc)[-1]
                length = modplug.ModPlug_GetLength(f) / 1000.0 or -1
                self.mutagen = {'title': name, '__length':length}
        else:
            self.mutagen = {'title':os.path.split(self.loc)[-1]}

    def get_length(self):
        try:
            return self.mutagen['__length']
        except:
            return -1

    def get_bitrate(self):
        return -1

# vim: et sts=4 sw=4

